use clap::{Args, ValueEnum};
use image::{DynamicImage, io::Reader as ImageReader};
use color_full16::{RgbaImage, RgbImage};
use std::io::Cursor;
use tool_helper::{Error, Input, Output};
use types::{Header, PSXImageConverter};

mod types;
mod color_full16;

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum)]
pub enum ColorDepth {
    Clut4,
    Clut8,
    Full16,
}

#[derive(Args)]
pub struct Arguments {
    #[clap(arg_enum, value_parser)]
    color_depth: ColorDepth
}


fn encode<T: PSXImageConverter>(image: T, mut output: Output) -> Result<(), Error> {
    let width  = image.width();
    let height = image.height();
    let header = Header::encode(width, height, 0, 0).ok_or(Error::from_callback(|| {format!("Image size {}, {} needs to be even", width, height)}))?;

    tool_helper::raw::write_generic(&mut output, header)?;
    for color in image {
        tool_helper::raw::write_generic(&mut output, color)?;
    }

    Ok(())
}

fn convert_full16(input: Input, output: Output) -> Result<(), Error> {
    match ImageReader::new(Cursor::new(tool_helper::input_to_vec(input)?)).with_guessed_format()?.decode() {
        Ok(image) => {
            match image {
                DynamicImage::ImageRgb8(image)  => encode(RgbImage::new(image), output),
                DynamicImage::ImageRgba8(image) => encode(RgbaImage::new(image), output),

                _ => Err(Error::from_text("Only RGB and RGBA images are supported for 16bit encoding".to_owned()))
            }
        },
        Err(error) => Err(Error::from_error(error))
    }
}

pub fn convert(args: Arguments, input: Input, output: Output) -> Result<(), Error> {
    match args.color_depth {
        ColorDepth::Full16 => convert_full16(input, output),
        _                  => Err(Error::not_implemented("Converting anything else then full16")),
    }
}